@@ -65,7 +65,7 @@ def create_logger(verbose=False):
 
         logHandler = logging.StreamHandler()
         if verbose:
-            logHandler.setFormatter(logging.Formatter('%(levelname)s: %(asctime)s #%(lineno)d - %(message)s', '%Y-%m-%d %H:%M'))
+            logHandler.setFormatter(logging.Formatter('%(levelname)s:\t%(asctime)s - %(message)s (line #%(lineno)d)', '%m-%d %H:%M'))
         else:
             logHandler.setFormatter(logging.Formatter('%(levelname)s: %(asctime)s - %(message)s', '%Y-%m-%d %H:%M'))
 
@@ -84,6 +84,13 @@ def get_password():
     """
     return getpass.getpass()
 
+def get_version():
+    """
+    Return the lcinvestor version number
+    """
+    this_path = os.path.dirname(os.path.realpath(__file__))
+    version_file = os.path.join(this_path, 'VERSION')
+    return open(version_file).read()
 
 def get_input(msg):
     """
